package jf01;

/*
 https://docs.oracle.com/javase/tutorial/java/nutsandbolts/datatypes.html
 https://beginnersbook.com/2017/08/data-types-in-java/
 */


/**
 *
 * @author Lefteris Moussiades
 */
public class Natives {

    static void byteDemo() { // 8-bit signed two's complement integer
        //0..2^7-1, -1..-2^7
        byte bMin = Byte.MIN_VALUE, bMax = Byte.MAX_VALUE;
        System.out.println("byte bMin equals to " + bMin);
        System.out.println("byte bMax equals to " + bMax);
        System.out.println("Size of byte is "+Byte.SIZE);
        int b;
        b=bMin-1; 
        System.out.println("bMin-1="+b);
        //b1=128;   //Compile Time Error
    }

    static void shortDemo() { // 16-bit signed two's complement integer
        short s = Short.MIN_VALUE, s1 = Short.MAX_VALUE;
        System.out.println("short s equals to " + s);
        System.out.println("short s1 equals to " + s1);
        System.out.println("Size of short is "+Short.SIZE);

        //s=Short.MIN_VALUE-1; s1=32768;   //Compile Time Error
    }

    static void intDemo() { // 32-bit signed two's complement integer
        int iMin = Integer.MIN_VALUE, 
            iMax = Integer.MAX_VALUE;
        int i;
        i=9;
        i=0b10011;
        System.out.println("i=0b10011 equals to "+i);
        i=076;
        i=0xA1;
        System.out.println("int iMin equals to " + iMin);
        System.out.println("int iMax equals to " + iMax);
        System.out.println("Size of int is "+Integer.SIZE);
        
        iMin = iMin-1;
        iMax = iMax+1;   //Overflow
        System.out.println("After Overflow");
        System.out.println("int iMin equals to " + iMin);
        System.out.println("int iMax equals to " + iMax);
    }

    static void longDemo() { // 64-bit signed two's complement integer
        long lMin = Long.MIN_VALUE,
             lMax = Long.MAX_VALUE;
        System.out.println("long lMin equals to " + lMin);
        System.out.println("long lMax equals to " + lMax);
        System.out.println("Size of long is "+Long.SIZE);

        //lMin = -92_233_720_368_547_758_809; //compile Error
        lMin=257_865_342;
        lMin-=1;
        lMax += 1;
        System.out.println("After overflow");
        System.out.println("long lMin equals to " + lMin);
        System.out.println("long lMax equals to " + lMax);
    }

    static void floatDemo() { // single-precision 32-bit IEEE 754 floating point
        float fMin = Float.MIN_VALUE, fMax = Float.MAX_VALUE;
        
        //float d=2.1; //Compile error
        float f2 = 2.1f;

        System.out.println("float fMin equals to " + fMin);
        System.out.println("float fMax equals to " + fMax);
        System.out.println("float f2 equals to " + f2);
        
        fMin=fMin-1; 
        fMax+=1;
        System.out.println("After Overflow");
        System.out.println("float fMin equals to " + fMin);
        System.out.println("float fMax equals to " + fMax);
        System.out.println(fMax==fMax+1);
        
        fMin = Float.NEGATIVE_INFINITY;
        fMax = Float.POSITIVE_INFINITY;
        float fByZero=f2/0;
        float nAN=(float)Math.sqrt(-1);

        System.out.println("float fMin equals to " + fMin);
        System.out.println("float fMax equals to " + fMax);
        System.out.println("float fByZero equals to " + fByZero);
        System.out.println("float nAN equals to " + nAN);
        
        System.out.println("Size of float is "+Float.SIZE);
    }
    
    static void doubleDemo() { // single-precision 64-bit IEEE 754 floating point
        double dMin = Double.MIN_VALUE, dMax = Double.MAX_VALUE;
        double d=2.1d;
        //double d = 2.1f;

        System.out.println("double dMin equals to " + dMin);
        System.out.println("double dMax equals to " + dMax);
        System.out.println("double d equals to " + d);
        
        dMin-=1;
        dMax+=1;
        System.out.println("After Overflow");
        System.out.println("double dMin equals to " + dMin);
        System.out.println("double dMax equals to " + dMax);
        System.out.println(dMax==dMax+1);

        dMin = Double.NEGATIVE_INFINITY;
        dMax = Double.POSITIVE_INFINITY;
        double dByZero=d/0;
        double nAN=Math.sqrt(-1);

        System.out.println("double dMin equals to " + dMin);
        System.out.println("double dMax equals to " + dMax);
        System.out.println("double dByZero equals to " + dByZero);
        System.out.println("double nAN equals to " + nAN);
        
            
        System.out.println("Size of double is " + Double.SIZE);
    }
    
    static void booleanDemo() {
        boolean b1=true, b2=false; //1 bit of information, not precise size, VM depended
        //boolean b3=0; Compile Error
        System.out.println("b1 equals to "+b1);
        System.out.println("b2 equals to "+b2);
    }
    
    static void charDemo() { //single 16-bit Unicode character
        char cMin=Character.MIN_VALUE, cMax=Character.MAX_VALUE;
        System.out.println("char cMin equals to "+cMin);
        System.out.println("char cMax equals to "+cMax);
        /*
        for (char c=Character.MIN_VALUE; c<=Character.MAX_VALUE; c++)
            System.out.print(c+" ");
*/
        System.out.println("");
        //cMin='a';
        System.out.println("Value of cMin is "+(int)cMin);
        System.out.println("Value of cMax is "+(int)cMax);
    }

    public static void main(String[] args) {
        //byteDemo();
        //shortDemo();
        //intDemo();

        //longDemo();
        floatDemo();
        //doubleDemo();
        
        //booleanDemo();
        //charDemo();
    }
}
